from textwrap import dedent
import tomlkit
import sys
import subprocess
import json
import platform
import typer

from pathlib import Path
from typing import Dict

from tomlkit.toml_document import TOMLDocument

from ..console import console


class LockFile:

    _MIN_EZCONDA_VERSION = "0.4.0"

    def __init__(self) -> None:

        self.doc = TOMLDocument()

    def _add_version_info(self) -> None:

        version = tomlkit.table()
        version.add("ezconda-min-version", self._MIN_EZCONDA_VERSION)

        self.doc.add("version", version)

    def _add_system_info(self) -> None:

        system = tomlkit.table()
        system.add("platform", sys.platform)
        system.add("architecture", platform.architecture()[0])
        system.add("machine", platform.machine())

        self.doc.add("system", system)

    def _add_current_packages(self, env_name: str) -> Dict:
        """Get current installed packages in the named environment"""

        p = subprocess.run(
            ["conda", "list", "-n", env_name, "--json"],
            capture_output=True,
            text=True,
        )

        # exit the app if error occurs
        if p.returncode != 0:
            console.print(f"[red]{str(p.stdout + p.stderr)}")
            raise typer.Exit()

        packages = json.loads(p.stdout)

        # store environment name
        _env_table = tomlkit.table()
        _env_table.add("name", env_name)

        self.doc.add("environment", _env_table)
        self.doc.add("packages", packages)

    def generate_lockfile(self, env_name: str) -> None:

        self._add_version_info()
        self._add_system_info()
        self._add_current_packages(env_name)

    def write_lockfile(self, env_name: str) -> None:

        self.generate_lockfile(env_name)

        lockfile_name = f"{env_name}-{sys.platform}-{platform.machine()}.lock"

        with open(lockfile_name, "w") as f:
            tomlkit.dump(self.doc, f)

        console.print(f"[bold green] :lock: Lock file '{lockfile_name}' generated")

    def _check_lock_file_exists(self, lock_file: Path):
        """If lock file does not exist, exit the app"""

        if not lock_file.is_file():
            console.print("[bold red]Lock file provided does not exist")
            raise typer.Exit()

    def read_lock_file(self, lock_file: Path) -> TOMLDocument:

        self._check_lock_file_exists(lock_file)

        with open(lock_file, "r") as f:
            doc = tomlkit.loads(f.read())
        return doc

    def verify_lock_file_contents(self, doc: TOMLDocument) -> None:

        # check contents
        if "version" not in doc or "system" not in doc:
            console.print(
                dedent(
                    """
                    [bold red]Lock file is missing fields and incorrect.[/]
                    
                    To correct and regenerate a new lock file:

                    [bold green]ezconda lock -n <environment_name>[/]

                    If the environment does not exist on your system,
                    you can create it using the corresponding '.yml' file:

                    [bold green]ezconda create -n <environment_name> -f <env.yml>[/]
                    """
                )  # TODO: add creating env from yml file
            )
            raise typer.Exit()

        # get system info from lock file
        system = doc["system"]

        # get current system info
        _current_system = {
            "platform": sys.platform,
            "architecture": platform.architecture()[0],
            "machine": platform.machine(),
        }

        if system != _current_system:
            console.print(
                dedent(
                    f"""
                    :heavy_exclamation_mark:[bold red]Incompatible lock file.[/]

                    Lock file was generated on [bold green]{system["platform"]}-{system["machine"]}[/].

                    Current system is [bold yellow]{_current_system["platform"]}-{_current_system["machine"]}[/].
                    """
                )
            )
            raise typer.Exit()



import json
import re
import subprocess
import yaml
import typer
from pathlib import Path
from typing import Optional, Dict, List
from textwrap import dedent

from .console import console


def create_initial_env_specs(
    env_name: str, channel: Optional[str] = None, packages: Optional[List[str]] = None
) -> Dict:
    """Create initial environment specifications that will be written to 'yml' file."""

    env_specs = {}
    env_specs.update({"name": env_name})
    if channel:
        env_specs.update({"channels": [channel, "defaults"]})
    else:
        env_specs.update({"channels": ["defaults"]})
    if packages:
        env_specs.update({"dependencies": packages})
    return env_specs


def get_validate_file_name(env_name: str, file: Optional[str] = None) -> Optional[str]:
    """
    Looks for a '.yml' file with the `env_name` specified. If file cannot
    be located, prompt will ask for the file name. If the file provided does
    not exist, the program will exit.
    """

    if not file:
        # first look for existing yml file
        if not Path(f"{env_name}.yml").is_file():
            console.print(f"[yellow]Couldn't locate {env_name}.yml")
            console.print(
                dedent(
                    f"""
            [yellow]If your environment name and specifications file name are not the same,
            please provide the specifications file name to update using the '-f' or '--file' flag.
            """
                )
            )
            raise typer.Exit()
        else:
            file = Path(f"{env_name}.yml")
    # validate the file that the user provides
    else:
        if not Path(file).is_file():
            console.print(f"[magenta]Could not locate '{file}'")
            raise typer.Exit()
    return file


def read_env_file(file: str) -> Dict:
    "Read '.yml' file and return a dict containing specifications in the file."

    with open(file, "r") as f:
        env_specs = yaml.load(f, Loader=yaml.FullLoader)
        return env_specs


def write_env_file(env_specs: Dict, file: str) -> None:
    "Writes '.yml' file based on the specifications provided."

    with open(file, "w") as f:
        yaml.safe_dump(env_specs, f, sort_keys=False)


def add_pkg_to_dependencies(env_specs: Dict, pkg_name: List[str]) -> Dict:
    """
    Checks if the package/s specified already exist in 'dependencies' section in 'yml' file.
    If package/s already exists, informs user and exits the program.
    If package/s does not exist, adds it to 'dependencies' section in 'yml' file.
    """

    existing_packages = env_specs.get("dependencies")

    # check if packages already exists
    if existing_packages:
        # create a list of existing packages without >,<,=
        existing_packages_re = [re.findall(r"\w+", d)[0] for d in existing_packages]

        for pkg in pkg_name:
            # strip any >,<,= from the package name that the user provided
            pkg_re = re.findall(r"\w+", pkg)[0]

            # exit if package already exists in the env.yml file
            if pkg_re in existing_packages_re:
                console.print(
                    f"[yellow]'{pkg_re}' already exists. Skipping installation.\n"
                    f"[yellow]If you want to update {pkg_re}, use `update` instead."
                )
                raise typer.Exit()

        env_specs["dependencies"] = existing_packages + list(pkg_name)
    else:
        env_specs["dependencies"] = list(pkg_name)
    return env_specs


def add_new_channel_to_env_specs(env_specs: Dict, channel: Optional[str]) -> Dict:
    """Add new channel to the environment specifications, if it does not exist."""

    # this should always return ["defaults"] atleast!
    if channel:
        existing_channels = list(env_specs.get("channels"))
        if existing_channels and channel not in existing_channels:
            existing_channels.append(channel)
            env_specs["channels"] = existing_channels
    return env_specs


def remove_pkg_from_dependencies(env_specs: Dict, pkg_name: List[str]) -> Dict:
    """
    Checks if the package/s specified already exist in 'dependencies' section in 'yml' file.
    If package/s does not exist, informs user and exits the program.
    If package/s exist, remove it from 'dependencies' section in 'yml' file.
    """

    existing_packages = env_specs.get("dependencies")

    # check if packages exists in specifications
    if existing_packages:
        # create a list of existing packages without >,<,=
        existing_packages_re = [re.findall(r"\w+", d)[0] for d in existing_packages]
        for pkg in pkg_name:
            # strip any >,<,= from the package name that the user provided
            pkg_re = re.findall(r"\w+", pkg)[0]

            # check if the package exists in the existing packages list
            if pkg_re not in existing_packages_re:
                console.print(
                    f"[bold red]'{pkg}' is not listed in '{env_specs['name']}.yml' file!"
                )
                raise typer.Exit()

            # this will only run if the package was found in the existing packages list
            for ext_pkg_re, ext_pkg in zip(existing_packages_re, existing_packages):
                if pkg_re == ext_pkg_re:
                    existing_packages.remove(ext_pkg)
                    existing_packages_re.remove(
                        ext_pkg_re
                    )  # need to remove it from this list as well otherwise zip will create an issue
                    env_specs["dependencies"] = existing_packages

    else:
        console.print(
            f"[bold red]There are no packages listed in '{env_specs['name']}.yml' file."
        )
        raise typer.Exit()
    return env_specs


def update_channels_after_removal(env_specs: Dict, env_name: str) -> Dict:
    """
    Updates channels in the environment specifications by looking at the exisiting channels in the environment.
    """

    # get list of channels
    p = subprocess.run(
        ["conda", "list", "-n", env_name, "--json"], capture_output=True, text=True
    )

    # identify unique ones and update channels in env_specs
    complete_dict: List[Dict] = json.loads(p.stdout)
    new_channels = list(set([d["channel"] for d in complete_dict]))
    new_channels.append("defaults")  # 'defaults' needs to be added back?
    env_specs["channels"] = new_channels
    return env_specs


def recheck_dependencies(env_specs: Dict, env_name: str) -> Dict:
    """
    Check if while removing a package, any dependent packages are also removed from env
    but not from .yml file. If so, remove them from .yml file
    """
    p = subprocess.run(
        ["conda", "list", "-n", env_name, "--json"], capture_output=True, text=True
    )
    complete_dict = json.loads(p.stdout)
    all_pkgs = set([d["name"] for d in complete_dict])

    deps = env_specs["dependencies"]  # this may have dependencies with ">,<,=" symbols
    deps_re = [re.findall(r"\w+", d)[0] for d in deps]  # removing the symbols

    rem_pkgs_to_be_removed_from_yml = set(deps_re) - all_pkgs
    if rem_pkgs_to_be_removed_from_yml:
        if "python" in rem_pkgs_to_be_removed_from_yml:
            rem_pkgs_to_be_removed_from_yml.remove("python")
        env_specs = remove_pkg_from_dependencies(
            env_specs, rem_pkgs_to_be_removed_from_yml
        )

    return env_specs



import typer

from . import __version__
from .console import console

# from .tree import show
from .create import create
from .install import install
from .remove import remove
from .recreate import recreate
from .config import config
from .summary import summary
from .experimental.lock import lock


app = typer.Typer(
    help="Create, Manage, Re-create conda environments & specifications with ease"
)

app.command()(create)
app.command()(install)
app.command()(remove)
app.command()(recreate)
app.command()(lock)
app.command()(summary)
app.command()(config)
# app.command()(show)


@app.command()
def version():
    """Shows the version of 'ezconda' installed"""
    console.print(f"[magenta]{__version__}")



import subprocess
import typer
from typing import List, Optional
from pathlib import Path

from .console import console
from ._utils import (
    get_validate_file_name,
    read_env_file,
    add_pkg_to_dependencies,
    write_env_file,
    add_new_channel_to_env_specs,
)
from .solver import Solver
from .config import get_default_solver
from .summary import get_summary_for_revision
from .experimental import write_lock_file


def install(
    pkg_name: List[str] = typer.Argument(..., help="Packages to install"),
    env_name: str = typer.Option(
        ...,
        "--name",
        "-n",
        prompt="Name of the environment to install in",
        help="Name of the environment to install package into",
    ),
    file: Optional[str] = typer.Option(
        None, "--file", "-f", help="'.yml' file to update with new packages"
    ),
    channel: Optional[str] = typer.Option(
        None, "--channel", "-c", help="Additional channel to search for packages"
    ),
    solver: Solver = typer.Option(None, help="Solver to use", case_sensitive=False),
    summary: bool = typer.Option(
        True, "--summary", help="Show summary of changes made"
    ),
    verbose: Optional[bool] = typer.Option(
        False, "--verbose", "-v", help="Display standard output from conda"
    ),
    lock: Optional[bool] = typer.Option(True, help="Write lockfile"),
):
    """
    Install package/s in specified conda environment.

    This command will also update the environment file and lock file.
    """

    with console.status(f"[magenta]Validating file, packages, channels") as status:

        file = get_validate_file_name(env_name, file)

        env_specs = read_env_file(file)
        env_specs = add_pkg_to_dependencies(env_specs, pkg_name)
        env_specs = add_new_channel_to_env_specs(env_specs, channel)

        if solver is None:
            solver = get_default_solver()

        status.update(f"[magenta]Resolving & Installing packages using {solver.value}")

        if not channel:
            p = subprocess.run(
                [f"{solver.value}", "install", "-n", env_name, *pkg_name, "-y"],
                capture_output=True,
                text=True,
            )
        else:
            p = subprocess.run(
                [
                    f"{solver.value}",
                    "install",
                    "-n",
                    env_name,
                    "--channel",
                    channel,
                    *pkg_name,
                    "-y",
                ],
                capture_output=True,
                text=True,
            )

        if p.returncode != 0:
            console.print(f"[red]{str(p.stdout + p.stderr)}")
            raise typer.Exit()

        if verbose:
            console.print(f"[yellow]{str(p.stdout)}")

        console.print(f"[bold green] :rocket: Installed packages in {env_name}")

        status.update(f"[magenta]Writing specifications to {file}")

        write_env_file(env_specs, file)
        console.print(f"[bold green] :floppy_disk: Updated specifications to '{file}'")

        if lock:
            status.update(f"[magenta]Writing lock file")

            write_lock_file(env_name)

        console.print(f"[bold green] :star: Done!")

        if summary:
            get_summary_for_revision(env_name)



import json
import subprocess
from textwrap import dedent
import typer

from rich.tree import Tree

from .console import console


def summary(
    name: str = typer.Option(
        ...,
        "--name",
        "-n",
        prompt="Name of the environment",
        help="Name of the environment to summarize",
    ),
    revision: int = typer.Option(
        -1,
        "--revision",
        help="Revision number to summarize; default is latest",
    ),
):
    _ = get_summary_for_revision(name, revision_no=revision)


def get_summary_for_revision(name: str, revision_no: int = -1):
    """ "
    Get summary for the revision number of the environment.
    By default, it will show the latest revision.
    """

    p = subprocess.run(
        ["conda", "list", "--revision", "-n", name, "--json"],
        capture_output=True,
        text=True,
    )

    # summary info
    _rev_data = json.loads(p.stdout)

    try:
        _info = _rev_data[revision_no]
    except IndexError:
        console.print(
            f"[red]Revision {revision_no} not found.\nYou have {len(_rev_data)} revisions for {name} environment."
        )
        raise typer.Exit(code=1)

    _installed = _info["install"]
    _upgraded = _info["upgrade"]
    _removed = _info["remove"]
    _downgraded = _info["downgrade"]

    console.print(
        dedent(
            f"""
            [bold green]{len(_installed)} Installs,[/] [bold purple]{len(_upgraded)} Upgrades[/], [bold yellow]{len(_downgraded)} Downgrades[/], [bold red]{len(_removed)} Removals[/]
            """
        )
    )

    tree = Tree("[bold]Summary[/]")
    if _installed:
        install_branch = tree.add("[bold green]Installs[/]")
        for i in _installed:
            install_branch.add(f"[green]{i}[/]")
    if _upgraded:
        upgrade_branch = tree.add("[bold purple]Upgrades[/]")
        for u in _upgraded:
            upgrade_branch.add(f"[purple]New: {u['new']}[/]\nOld: {u['old']}")
    if _downgraded:
        downgrade_branch = tree.add("[bold yellow]Downgrades")
        for d in _downgraded:
            downgrade_branch.add(f"[yellow]New: {d['new']}[/]\nOld: {d['old']}")
    if _removed:
        removal_branch = tree.add("[bold red]Removals[/]")
        for r in _removed:
            removal_branch.add(f"[red]{r}[/]")

    if _installed or _upgraded or _downgraded or _removed:
        console.print(tree)
    
    return _installed, _upgraded, _downgraded, _removed



import subprocess
from textwrap import dedent
import typer
from typing import List, Optional
from pathlib import Path

from .console import console
from ._utils import create_initial_env_specs, write_env_file, read_env_file
from .solver import Solver
from .summary import get_summary_for_revision
from .experimental import write_lock_file
from .config import get_default_solver


def create(
    name: str = typer.Option(
        None,
        "--name",
        "-n",
        help="Name of the environment to create",
    ),
    packages: List[str] = typer.Argument(None, help="Additional packages to install"),
    channel: Optional[str] = typer.Option(
        None, "--channel", "-c", help="Additional channel to search for packages"
    ),
    solver: Solver = typer.Option(None, help="Solver to use", case_sensitive=False),
    file: Optional[Path] = typer.Option(
        None, "--file", "-f", help="Name of the environment yml file"
    ),
    summary: bool = typer.Option(
        True, "--summary", help="Show summary of changes made"
    ),
    verbose: Optional[bool] = typer.Option(
        False, "--verbose", "-v", help="Display standard output from conda"
    ),
    lock: Optional[bool] = typer.Option(True, help="Generate and write lockfile"),
):
    """
    Create new conda environment with a corresponding environment file and 'lock' file.

    Environment file contains environment specifications and 'lock' file contains complete
    specifications for reproducible environment builds.
    """

    # create a yml file to write specs to
    if file is None:
        if name is None:
            name = typer.prompt("Name of the environment to create")

        file = Path(f"{name}.yml")

        # check if file (and likely environment) already exists
        if file.is_file():
            overwrite = typer.confirm(
                dedent(
                    f"""
                There is an existing {file} file.

                Please note that this likely means you have an existing conda environment with the same name as {name}.
                
                Do you want to update the file and environment?
                
                Answering "Yes"/"y" will create a new conda environment '{name}' and overwrite '{file}'.
                """,
                ),
                abort=True,
            )
            console.print(f"[bold yellow]Overwrote {file} ...")

        env_specs = create_initial_env_specs(name, channel, packages)

        if solver is None:
            solver = get_default_solver()

        with console.status(
            f"[magenta]Creating new conda environment {name}"
        ) as status:

            if packages:
                status.update(
                    status=f"[magenta]Resolving & Installing packages using {solver.value}"
                )

            if not channel:
                p = subprocess.run(
                    [f"{solver.value}", "create", "-n", name, *packages, "-y"],
                    capture_output=True,
                    text=True,
                )

            else:
                p = subprocess.run(
                    [
                        f"{solver.value}",
                        "create",
                        "-n",
                        name,
                        "--channel",
                        channel,
                        *packages,
                        "-y",
                    ],
                    capture_output=True,
                    text=True,
                )

            if p.returncode != 0:
                console.print(f"[red]{str(p.stdout + p.stderr)}")
                raise typer.Exit()

            if verbose:
                console.print(f"[bold yellow]{str(p.stdout)}")

            console.print(f"[bold green] :rocket: Created '{name}' environment")

            status.update(f"[magenta]Writing specifications to {file}")
            write_env_file(env_specs, file)
            console.print(
                f"[bold green] :floppy_disk: Saved specifications to '{file}'"
            )

            if (
                lock and packages
            ):  # only write lock file if packages are mentioned during env creation
                status.update(f"[magenta]Writing lock file")

                write_lock_file(name)

    elif file.is_file():
        with console.status(
            f"[magenta]Creating new conda environment using {file}"
        ) as status:

            p = subprocess.run(
                ["conda", "env", "create", "--file", file],
                capture_output=True,
                text=True,
            )

            if p.returncode != 0:
                console.print(f"[red]{str(p.stdout + p.stderr)}")
                raise typer.Exit()

            # get env name from yml file
            name = read_env_file(file)["name"]
            console.print(f"[bold green] :rocket: Created '{name}' environment")

            if (
                lock
            ):  # only write lock file if packages are mentioned during env creation
                status.update(f"[magenta]Writing lock file")
                write_lock_file(name)

            if verbose:
                console.print(f"[bold yellow]{str(p.stdout)}")

    console.print(f"[bold green] :star: Done!")

    if summary:
        get_summary_for_revision(name)



import subprocess
import typer
import time
import json
import os
import conda.exports

from typing import List, Optional

from .console import console
from ._utils import (
    get_validate_file_name,
    read_env_file,
    remove_pkg_from_dependencies,
    write_env_file,
    update_channels_after_removal,
    recheck_dependencies,
)
from .solver import Solver
from .config import get_default_solver
from .experimental import write_lock_file


def remove(
    pkg_name: List[str] = typer.Argument(..., help="Packages to uninstall"),
    env_name: str = typer.Option(
        ...,
        "--name",
        "-n",
        prompt="Name of the environment to remove from",
        help="Name of the environment to uninstall package from",
    ),
    solver: Solver = typer.Option(None, help="Solver to use", case_sensitive=False),
    file: Optional[str] = typer.Option(
        None, "--file", "-f", help="'.yml' file to update with removed packages"
    ),
    verbose: Optional[bool] = typer.Option(
        False, "--verbose", "-v", help="Display standard output from conda"
    ),
    lock: Optional[bool] = typer.Option(True, help="Write lockfile"),
):
    """
    Remove/uninstall packages from environment.

    This command will also update the environment file and lockfile.
    """

    with console.status(f"[magenta]Validating file, packages, channels") as status:
        file = get_validate_file_name(env_name, file)

        installed_packages = [
            specs["name"]
            for specs in json.load(os.popen(f"conda list -n {env_name} --json"))
        ]
        root_prefix = json.load(os.popen(f"conda info -e --json"))["root_prefix"]
        linked_data = conda.exports.linked_data(root_prefix)

        other_pkg_that_depends_on_pkg = []
        for k in linked_data.keys():
            if linked_data[k]["name"] in installed_packages:
                # check the dependencies
                # if the package listed for removal is listed as dependency for any other installed package
                # let the user know before they confirm to the removal
                for deps in linked_data[k]["depends"]:
                    for pkg in pkg_name:  # unpack multiple packages
                        if pkg in deps.split(" "):
                            other_pkg_that_depends_on_pkg.append(linked_data[k]["name"])

        if other_pkg_that_depends_on_pkg:
            console.print(f"[magenta]There are packages that depend on {pkg_name}")
            console.print(
                f"[magenta]Removing {pkg_name} will also remove the following:"
            )
            console.print(f"[magenta]{other_pkg_that_depends_on_pkg}\n")
            status.stop()
            typer.confirm(f"Do you want to continue?", abort=True)
            status.start()

        env_specs = read_env_file(file)
        env_specs = remove_pkg_from_dependencies(env_specs, pkg_name)

        status.update("[magenta]Removing packages")

        if solver is None:
            solver = get_default_solver()

        p = subprocess.run(
            [
                f"{solver.value}",
                "remove",
                "-n",
                env_name,
                *pkg_name,
                "-y",
            ],
            capture_output=True,
            text=True,
        )

        if p.returncode != 0:
            console.print(f"[red]{str(p.stdout + p.stderr)}")
            raise typer.Exit()

        if verbose:
            console.print(f"[yellow]{str(p.stdout)}")

        env_specs = update_channels_after_removal(env_specs, env_name)

        # check if any dependent packages are removed from env but not from .yml file
        # if so, remove them from .yml file
        env_specs = recheck_dependencies(env_specs, env_name)

        console.print(f"[bold green] :rocket: Removed packages from {env_name}")

        status.update(f"[magenta]Writing specifications to {file}")
        write_env_file(env_specs, file)
        console.print(f"[bold green] :floppy_disk: Updated specifications in '{file}'")

        if lock:
            status.update(f"[magenta]Writing lock file")

            write_lock_file(env_name)

        console.print(f"[bold green] :star: Done!")



import subprocess
import tempfile
import typer
from typing import Optional
from pathlib import Path

from .console import console
from .solver import Solver
from .config import get_default_solver
from .files.lockfile import LockFile


def read_lock_file_and_install(
    lock_file: Path,
    solver: Solver,
    verbose: bool,
    env_name: Optional[str] = None,
) -> None:
    """
    Reads lock file, verifies the content and creates a new environment
    with packages specifed in the lock file.
    """

    with console.status(f"[magenta]Reading lock file") as status:

        l = LockFile()
        complete_specs = l.read_lock_file(lock_file)

        l.verify_lock_file_contents(complete_specs)

        if env_name is None:
            env_name = complete_specs["environment"]["name"]

        explicit_specs = [
            f"{pkg['base_url']}/{pkg['platform']}/{pkg['dist_name']}"
            for pkg in complete_specs["packages"]
            if not pkg["channel"].startswith("pypi")
        ]

        explicit_specs = [
            spec + ".conda"
            if spec.startswith("https://repo.anaconda.com")
            else spec + ".tar.bz2"
            for spec in explicit_specs
        ]

        with tempfile.NamedTemporaryFile() as f:
            f.write(bytes("@EXPLICIT\n", "utf-8"))
            f.writelines([bytes(specs + "\n", "utf-8") for specs in explicit_specs])
            f.flush()

            status.update(f"[magenta]Creating new conda environment {env_name}")

            p = subprocess.run(
                [
                    f"{solver.value}",
                    "create",
                    "-n",
                    env_name,
                    "--file",
                    f"{f.name}",
                    "-y",
                ],
                capture_output=True,
                text=True,
            )

            if verbose:
                console.print(f"[bold yellow]{p.stdout}")

            if p.returncode != 0:
                console.print("[red]" + str(p.stdout + p.stderr))
                raise typer.Exit()

        # TODO: Add installation for pypi channels/packages

        console.print(
            f"[bold green] :rocket: Recreated '{env_name}' environment from lock file",
        )


def recreate(
    file: Path = typer.Argument(
        ..., help="Lock file to use to re-create an environment"
    ),
    name: Optional[str] = typer.Option(
        None,
        "--name",
        "-n",
        help="Name of the environment to create",
    ),
    solver: Solver = typer.Option(None, help="Solver to use", case_sensitive=False),
    verbose: Optional[bool] = typer.Option(
        False, "--verbose", "-v", help="Display standard output from conda"
    ),
):
    """
    Re-create an environment from lock file. This will install all the packages
    specified in the lock file, if the system and platform match lock file.
    """

    if solver is None:
        solver = get_default_solver()

    read_lock_file_and_install(file, solver, verbose, name)



import typer
from pathlib import Path
from typing import Optional, Dict
import tomlkit

from .solver import Solver

from .console import console


# get ezconda config file
app_dir = typer.get_app_dir("ezconda")
config_file: Path = Path(app_dir) / "config.toml"


def check_configs(
    app_dir: str = app_dir, config_file: Path = config_file
) -> Optional[Dict]:
    """
    Check app config file for user settings.
    Retruns a dict of configs if they exist.
    """

    # check if directory exists
    if Path(app_dir).is_dir():
        # check if config file exists
        if config_file.is_file():
            with open(config_file, "r") as f:
                configs = tomlkit.load(f)
            return configs


def make_and_read_config_file(
    app_dir: str = app_dir, config_file: Path = config_file
) -> Dict:
    """Create config file if it doesn't exist and return empty dict of configs.
    If one exists, read it and return a dict of configs."""

    # create app directory if it doesn't exist
    if not Path(app_dir).is_dir():
        Path(app_dir).mkdir(parents=True)

    # create config file if it doesn't exist
    # this will happen the first time config is run
    if not config_file.is_file():
        config_file.touch()
        configs = {}

    else:
        with open(config_file, "r") as f:
            configs = tomlkit.load(f)

    return configs


def get_default_solver() -> Solver:
    """Checks config file for default solver.
    If config file or solver related config doesn't exist,
    returns mamba as default solver.
    """
    config = check_configs()

    if config:
        if config.get("solver") is not None:
            solver = config["solver"]
        else:
            solver = Solver.mamba
    else:
        solver = Solver.mamba
    return solver


def config(
    solver: Solver = typer.Option(
        None,
        "--solver",
        help="Set default solver",
        case_sensitive=False,
    ),
    show: bool = typer.Option(False, "--show", help="Show current config"),
):
    """
    Configure default settings for ezconda
    """
    configs = make_and_read_config_file()

    if solver:
        configs.update({"solver": solver})

    with open(config_file, "w") as f:
        tomlkit.dump(configs, f)

    if show:
        console.print(f"[bold green]Current Configuration")
        console.print_json(data=configs)



# import typer
# from rich.tree import Tree
# import conda.exports
# import conda.api
# import os
# import json

# from .console import console


# def create_dep_graph(pkg, linked_data, color="green"):
#     # root_prefix = json.load(os.popen(f"conda info -e --json"))["root_prefix"]
#     # linked_data = conda.exports.linked_data(root_prefix)

#     root = Tree(f"[bold {color}]{pkg}")
#     for k in linked_data.keys():
#         if linked_data[k]["name"] == pkg.split(" ")[0]:
#             # check the dependencies
#             for deps in linked_data[k]["depends"]:
#                 # root.add(f"[magenta]{deps}")
#                 root.add(create_dep_graph(deps, linked_data, color="yellow"))

#     return root


# def show(
#     pkg_name: str = typer.Argument(
#         ..., help="Name of the package to show dependencies"
#     ),
# ):
#     root_prefix = json.load(os.popen(f"conda info -e --json"))["root_prefix"]
#     linked_data = conda.exports.linked_data(root_prefix)
#     console.print(create_dep_graph(pkg_name, linked_data))



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ezconda', 'ezconda.experimental', 'ezconda.files']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=5.4.1', 'rich>=10.11.0', 'tomlkit>=0.9.0', 'typer[all]>=0.4.0']

entry_points = \
{'console_scripts': ['ezconda = ezconda.main:app']}

setup_kwargs = {
    'name': 'ezconda',
    'version': '0.6.0',
    'description': 'Create, Manage, Re-create conda environments & specifications with ease.',
    'long_description': '# EZconda\n\n![EZconda](https://github.com/SarthakJariwala/ezconda/blob/2945291bc9ef123cb52e9c6436906ac0728b0451/docs/logo.png)\n\n<p align="center">\n    <a href="https://github.com/SarthakJariwala/ezconda/actions?workflow=Tests">\n        <img src="https://github.com/SarthakJariwala/ezconda/workflows/Tests/badge.svg">\n    </a>\n    <a href="https://codecov.io/gh/SarthakJariwala/ezconda">\n        <img src="https://codecov.io/gh/SarthakJariwala/ezconda/branch/main/graph/badge.svg">\n    </a>\n    <a href="https://anaconda.org/conda-forge/ezconda">\n        <img alt="Conda (channel only)" src="https://img.shields.io/conda/vn/conda-forge/ezconda">\n    </a>\n    <a href="https://ezconda.sarthakjariwala.com">\n        <img src="https://github.com/SarthakJariwala/ezconda/workflows/Docs/badge.svg">\n    </a>\n</p>\n\n<p align="center">\n    <em><b>Create, Manage, Re-create</b> conda environments & specifications with ease.</em>\n</p>\n\n---\n\n**EZconda** is a command line interface application that helps practitioners create and manage `conda` environment and related specifications with ease.\n\n## Key Features\n\n- **Environment specifications** : Add & remove packages from the <abbr title="commonly known as environment.yml file">specifications file</abbr> as you install & remove packages. _**No manual file edits!**_\n\n- **Environment management** : Create & manage `conda` environments with ease.\n\n- **Reproducible environments** : Lock current environment state and re-create it when necessary.\n\n- **Easy to use & intuitive** : It very closely mimics `conda` API, so there is no new API to learn for users. Autocomplete for all shells.\n\n- **Fast & Reliable Environment resolution** : Get fast and reliable environment solves by default. *EZconda* uses `mamba` by default, but you can easily switch between `mamba` and `conda`.\n\n- **Best practices built-in** : Enforces the user to follow best `conda` practices.\n\n## Requirements\n\n- [Conda](https://docs.conda.io/projects/conda/en/latest/user-guide/install/index.html) installation\n\n## Installation\n\nThe recommended way to install **EZconda** is using `conda` or `mamba` in the `base` environment : \n\n### Using `conda`: \n\n```console\n$ conda install ezconda -c conda-forge -n base\n```\n\n### Using `mamba`:\n\n```console\n$ mamba install ezconda -c conda-forge -n base\n```\n\n## Contributing Guidelines\n\n<!-- TODO Add contributing guidelines -->\n\n### Run tests\n\n```bash\ndocker-compose up --build test\n```\n\n### Local iterative development\n\n```bash\ndocker-compose build dev && docker-compose run dev bash\n```',
    'author': 'Sarthak Jariwala',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import typer
from ..console import console
from ..files.lockfile import LockFile


__all__ = ["write_lock_file", "lock"]


def write_lock_file(env_name: str) -> None:
    """
    Writes a lock file for the environment specified.
    """

    lockfile = LockFile()
    lockfile.write_lockfile(env_name)


def lock(
    env_name: str = typer.Option(
        ...,
        "--name",
        "-n",
        prompt="Name of the environment",
        help="Name of the environment to generate lock file for",
    ),
) -> None:
    """
    Generate lock file for a conda environment
    """
    with console.status(f"[magenta]Writing lock file"):
        write_lock_file(env_name)

    console.print(f"[bold green] :star: Done!")
